/**
******************************************************************************
* @file    message_interface_common.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief Common functions for the message generator and the message interpreter.
*
* @verbatim
 ===============================================================================
                      ##### Description #####
 ===============================================================================
  [..]
  Routines to access channel_id mapping into physical sensors populated on the board
  given
        +-----+-----+-----+-----+-----+    which assigns a unique Id to each channel
        | ch  |  u1 |  u2 |  u3 | u4  |    a logic is implemented to manage the mapping
        +-----+-----+-----+-----+-----+    into the IRPyro API
        |  0  |0x01 |0x06 |0x0b |0x10 |
        +-----+-----+-----+-----+-----+
        |  1  |0x02 |0x07 |0x0c |0x11 |
        +-----+-----+-----+-----+-----+
        |  2  |0x03 |0x08 |0x0d |0x12 |
        +-----+-----+-----+-----+-----+
        |  3  |0x04 |0x09 |0x0e |0x13 |
        +-----+-----+-----+-----+-----+
        |  4  |0x05 |0x0a |0x0f |0x14 |
        +-----+-----+-----+-----+-----+
  [..]
 @endverbatim
  ******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "message_interface_common.h"

/** @defgroup message_interface_common Message Interface Common
  * @ingroup message_interface
  * @brief Functions common to Message Generator and Message Interpreter
  * @verbatim
  *
  * @endverbatim
  * @note
  * @{
  */
/**
* @brief check for channels. returns an array with the channels and count
*/
static uint8_t channel_collect(uint8_t *channel_id)
{
    uint8_t channel_count;
    uint8_t channels_available;
    for(channel_count = 0, channels_available=0; channels_available < NUMBER_OF_SCOPES_IN_THE_GUI; ++channels_available)
    {
        if (ChannelConfiguration[channels_available])
        {
            channel_id[channel_count] = ChannelConfiguration[channels_available];
            ++ channel_count;
        }
    }
    return channel_count;
}
/**
* @brief looks for a given channel id on the list of active channels
* @note returns the channel ordinal not the array index: 1st, 2nd, etc
*/
static uint8_t channel_locate(const uint8_t *channels)
{
    uint8_t op_result;
    uint8_t *channel_found = NULL;
    if (sensor_destination != 0x2A)
    {
        channel_found = memchr(channels, ChannelConfiguration[sensor_destination], NUMBER_OF_SCOPES_IN_THE_GUI);
    }
    if (channel_found != NULL)
    {
        op_result = channel_found - channels +1;
    }
    else
    {
        op_result = 0;
    }
    return op_result;
}
/**
  * @brief validation logic for available channels, sensor selected and channel location
  */
static uint8_t channel_validate(uint8_t *idx, uint8_t *channel_count, uint8_t sensor_selected, uint8_t channel_location)
{
    uint8_t op_result;
    if (*channel_count)
    {
        if (sensor_selected == 0x2a)
        {
            *idx = 0;
            op_result = 1;
        }
        else
        {
            if(channel_location)
            {
                op_result = 1;
                -- channel_location; // decrement to use as array index
                *idx = channel_location;
                *channel_count = 1;
            }
            else
            {
                op_result = 0;
            }
        }
    }
    else
    {
        op_result = 0;
    }
    return op_result;
}
/**
  * @brief configures for loop for single or multiple sensor operations
  */
void process_parms_load(uint8_t *idx, uint8_t *num_sensors_to_process)
{
    if (sensor_destination == 0x2A)
    {
        *idx =0;
        *num_sensors_to_process = num_of_IRPyro_on_demo_board;
    }
    else
    {
        *idx = sensor_destination;
        *num_sensors_to_process = 1;
    }
}
/**
 * @struct channel_configuration_map message_interface_common.c
 * @brief assign a unique consecutive number to each channel on a maximum of four sensors
 *
 * The eevt gui has a maximum of four scopes to receive collected data from a configured
 * channel. In order to present
*/
struct channel_configuration_map
{
    uint8_t index;
    uint8_t map_sensor_selected;
    uint8_t map_channel_selected;
};
const struct channel_configuration_map on_board[] =
{
    {0x00,0,0},
    {0x01,0,0},{0x02,0,1},{0x03,0,2},{0x04,0,3},{0x05,0,4},
    {0x06,1,0},{0x07,1,1},{0x08,1,2},{0x09,1,3},{0x0A,1,4},
    {0x0B,2,0},{0x0C,2,1},{0x0D,2,2},{0x0E,2,3},{0x0F,2,4},
    {0x10,3,0},{0x11,3,1},{0x12,3,2},{0x13,3,3},{0x14,3,4}
};
static uint8_t channels[NUMBER_OF_SCOPES_IN_THE_GUI] = {0};
static uint8_t channel_processing_index = 0;
/**
  * @brief prepares for channel_id requests
  */
uint8_t channel_processing(uint8_t *channel_count)
{
    uint8_t channel_location = 0;
    uint8_t channel_ok       = 0;
    channel_processing_index = 0;
    memset(channels, 0x00, NUMBER_OF_SCOPES_IN_THE_GUI);

    *channel_count   = channel_collect(channels);
    channel_location = channel_locate(channels);
    channel_ok       = channel_validate(&channel_processing_index, channel_count, sensor_destination, channel_location);

    return channel_ok;
}
/**
  * @brief  maps the GUI scope channel_id to physical sensor
  */
uint8_t channel_id_to_sensor(void)
{
    uint8_t op_result = on_board[channels[channel_processing_index]].map_sensor_selected;
    return op_result;
}
/**
  * @brief maps the GUI scope channel_id to physical channel on the sensor
  */
uint8_t channel_id_to_channel(void)
{
    uint8_t op_result = on_board[channels[channel_processing_index]].map_channel_selected;
    return op_result;
}
/**
  * @brief increments the channel index
  */
void channel_id_next(void)
{
    ++channel_processing_index;
}
/**
* @}
*/
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
