/**
******************************************************************************
* @file    message_interpreter.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief Decoded commands from the GUI are prepared for application to the sensor
*
* @verbatim
 ===============================================================================
                      ##### Description #####
 ===============================================================================
  [..]
  given a valid instruction code message_interpreter()
   - passes the code of the acted command
   - sets the next state of eevt: apply parameters or contact_GUI
   - SET case: fills IRPyro_device_cmd and IRPyro_device_arg with the passed values
   - GET case: flags message_to_GUI to recover the proper values.
  [..]
 @endverbatim
  ******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#ifndef MESSAGE_TO_SENSOR
#include "message_interpreter.h"
#endif
/** @defgroup message_interpreter Message Interpreter
  * @ingroup message_interface
  * @brief Given a byte string response from the firmware, selects the adequate frame for the GUI.
  * @verbatim
  *
  * @endverbatim
  * @note
  * @{
  */
uint8_t message_to_sensor_payload[7] = {0};
execute_next_state_type next_step;
static void report_error(void);
extern EXTernal_callback pointer_to_read_function;
extern EXTernal_callback pointer_to_wakeup_function;

/**
  * @brief  decodes the AFE register of the IRPyro into this_device
  */
static void bytes_to_AFE(uint8_t *AFE_bytes, AFE_reg_type *AFE_register)
{
    AFE_register->S7_S0   = AFE_bytes[0];              // 0b11111111
    AFE_register->LP      = (AFE_bytes[1]& 0X80) >> 7; // 0b10000000
    AFE_register->HP      = (AFE_bytes[1]& 0X40) >> 6; // 0b01000000
    AFE_register->C_LP    = (AFE_bytes[1]& 0X30) >> 4; // 0b00110000
    AFE_register->CLK_OUT = (AFE_bytes[1]& 0X08) >> 3; // 0b00001000
    AFE_register->SYNC    = (AFE_bytes[1]& 0X04) >> 2; // 0b00000100
    AFE_register->TEMP    = (AFE_bytes[1]& 0X02) >> 1; // 0b00000010
    AFE_register->INT     = (AFE_bytes[1]& 0X01) >> 0; // 0b00000001
}
/**
  * @brief  decodes the CCP register of the IRPyro into this_device
  */
static void bytes_to_CCP(uint8_t CCP_byte, CCP_reg_type *CCP_register)
{
    CCP_register->Feedback_transconductance = (CCP_byte & 0XC0 )>> 6 ; // 0b11000000
    CCP_register->High_pass_filter_setting  = (CCP_byte & 0X30 )>> 4 ; // 0b00110000
    CCP_register->Feedback_cap              = (CCP_byte & 0X0E )>> 1 ; // 0b00001110
    CCP_register->status_bit                = (CCP_byte & 0X01 )>> 0 ; // 0b00000001
}
/**
  * @brief  decodes the WUP register of the IRPyro into this_device
  */
static void bytes_to_WUP(uint8_t *WUP_bytes, WUP_reg_type *WUP_register)
{
    WUP_register->UHT          = WUP_bytes[0];
    WUP_register->ULT          = WUP_bytes[1];
    WUP_register->LHT          = WUP_bytes[2];
    WUP_register->LLT          = WUP_bytes[3];
    WUP_register->WT_Threshold = WUP_bytes[4];
    WUP_register->ST           =(WUP_bytes[5] & 0X40) >> 6; // 0b01000000
    WUP_register->DP0_DP2      =(WUP_bytes[5] & 0X38) >> 3; // 0b00111000
    WUP_register->CH0_CH2      =(WUP_bytes[5] & 0X07) >> 0; // 0b00000111
}
/**
  * @brief  configures the null test
  */
static void null_test_command(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief  configures firmware version string
  */
static void firmware_version_send(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void asic_version_read(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void intpr_AFE_read(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief commits the AFE values to the structure in memory
  */
static void intpr_AFE_write(void)
{
#define AFE_SIZE (2)
    uint8_t afe_bytes[AFE_SIZE];
    uint8_t sensor_selected;
    uint8_t payload_index = 0;
    uint8_t num_sensors_to_process;

    process_parms_load(&sensor_selected, &num_sensors_to_process);
    do
    {
        afe_bytes[0] = IRPyro_device[sensor_selected].AFE_register.S7_S0;
        afe_bytes[1] = message_to_sensor_payload[payload_index];
        if (message_to_sensor_payload[payload_index] & 0x01)
            IRPyro_device[sensor_selected].read_mode = 0;
        else
            IRPyro_device[sensor_selected].read_mode = 1;
        bytes_to_AFE(afe_bytes, &IRPyro_device[sensor_selected].AFE_register);
        ++ sensor_selected;
        ++ payload_index;
        --num_sensors_to_process;
    }
    while(num_sensors_to_process);
    IRPyro_device_cmd.cmd = register_set;
    next_step = execute_apply_parameters;
}
/**
  * @brief Collects values stored in the channels configured on channel configuration
  */
static void intpr_CCP_read(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Writes CCP values to channels configured on channel configuration
  * the CCP values are given as single registers by the GUI. they need to be
  * converted to API form and written to the registers in memory. The values in
  * memory are committed to the actual sensor using the ioctl command
  */
static void intpr_CCP_write(void)
{
    uint8_t channel_count = 0;
    uint8_t channel_ok = channel_processing(&channel_count);
    if (channel_ok)
    {
        uint8_t payload_idx = 0;
        do
        {
            CCP_reg_type CCP_value = {0};

            bytes_to_CCP(message_to_sensor_payload[payload_idx], &CCP_value);
            IRPyro_device[channel_id_to_sensor()].CCP_register[channel_id_to_channel()] = CCP_value;
            channel_id_next();

            ++ payload_idx;
            -- channel_count;
        }
        while (channel_count);
        IRPyro_device_cmd.cmd = register_set;
        next_step = execute_apply_parameters;
    }
    else
    {
        report_error();
    }
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void intpr_WUP_read(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Loads the WUP values to be applied to the sensor
  */
static void intpr_WUP_write(void)
{
    bytes_to_WUP(message_to_sensor_payload, &IRPyro_device[sensor_destination].WUP_register);
    IRPyro_device_cmd.cmd = register_set;
    next_step = execute_apply_parameters;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void streaming_start(void)
{
    next_step = execute_contact_gui;
    isStreaming = 1;
}
/**
  * @brief Null operation for the state machine
  */
static void fndummy(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Configures the wake up function and sends the sleep command
  */
static void goto_sleep(void)
{
    IRPyro_device_cmd.cmd = power_sleep;
    mode_sleeping = 1;
    isStreaming   = 0;
    EXTernal_Callback_Setup(pointer_to_wakeup_function);
    next_step = execute_apply_parameters;
}
/**
  * @brief Wakes up the sensor and disables the call back function
  */
static void wake_up(void)
{
    IRPyro_device_cmd.cmd = power_wakeup;
    mode_sleeping = 0;
    isStreaming   = 1;
    EXTernal_Callback_Setup(NULL);
    next_step = execute_apply_parameters;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void reset_soft(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void reset_hard(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void reset_hardware(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void sampling_rate_read(void)
{
    next_step = execute_contact_gui;
}

/**
  * @brief Configures the sample rate in memory and prepares message to sensor
  */
static void sampling_rate_write(void)
{
    IRPyro_device_cmd.cmd = register_set;
    IRPyro_device[0].AFE_register.S7_S0 = message_to_sensor_payload[0];
    IRPyro_device[1].AFE_register.S7_S0 = message_to_sensor_payload[0];
    IRPyro_device[2].AFE_register.S7_S0 = message_to_sensor_payload[0];
    IRPyro_device[3].AFE_register.S7_S0 = message_to_sensor_payload[0];
    next_step = execute_apply_parameters;
}

/**
  * @brief Returns the result to the operation to the GUI
  */
static void logical_channels_read(void)
{
    next_step = execute_contact_gui;
}

/**
  * @brief Updates with values received from the GUI
  */
static void logical_channels_write(void)
{
    for (uint8_t index=0; index< NUMBER_OF_SCOPES_IN_THE_GUI; ++index)
    {
        ChannelConfiguration[index] = message_to_sensor_payload[index];
    }
    next_step = execute_contact_gui;
}
/**
  * @brief Updates with the values received from the GUI
  */
static void set_emitter_timings(void)
{
    IRPyro_device_cmd.cmd = register_get;
    EmitterONTime  = message_to_sensor_payload[0] ;
    EmitterOFFTime = message_to_sensor_payload[1] ;
    next_step = execute_contact_gui;
}
/**
  * @brief Updates with values received from the GUI
  */
static void set_emitter_state(void)
{
    EmitterStatus = message_to_sensor_payload[0] | 0x80;
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void get_failure_flags(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void board_type_read(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void unit_id_read(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void unit_id_write(void)
{
    IRPyro_device_cmd.cmd = register_get;
    UnitID[0] = message_to_sensor_payload[0];
    UnitID[1] = message_to_sensor_payload[1];
    next_step = execute_apply_parameters;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void second_unit_id_read(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void get_darkpixel_state(void)
{
    next_step = execute_contact_gui;
}
/**
  * @brief Returns the result to the operation to the GUI
  */
static void set_darkpixel_state(void)
{
    if (DarkPixelState)
        DarkPixelState = 0;
    else
        DarkPixelState = 1;
    next_step = execute_contact_gui;
}
/**
  * @brief Changes the streaming flag and reports to the GUI
  */
static void streaming_stop(void)
{
    isStreaming = 0;
    next_step = execute_contact_gui;
}
/**
  * @brief Changes the error flag and reports to the GUI
  */
static void report_error(void)
{
    error_flag = 1;
    next_step = execute_contact_gui;
}
/**
  * @brief Enables status report to the GUI
  */
static void status_start(void)
{
    isStatus = 1;
    next_step = execute_contact_gui;
}
/**
  * @brief Disables status report to the GUI
  */
static void status_stop(void)
{
    isStatus = 0;
    next_step = execute_contact_gui;
}

/**
 * @brief Recovers the next message from the GUI to the sensor
 * If the message is valid, is passed to the corresponding function.
 * the function then sets flags or loads parameters.
 *
*/
execute_next_state_type message_interpreter(uint8_t *cmd_code, uint8_t *destination)
{
#define CMD_ARRAY_SIZE (35)
    uint8_t cmd_tab_idx;
    static void (* const pf[CMD_ARRAY_SIZE])(void) =
    {
        //                                   CODE    CHECK
        null_test_command,           //  0 - 0x00    0xFF
        firmware_version_send,       //  1 - 0x01    0xFE
        asic_version_read,           //  2 - 0x02    0xFD
        intpr_CCP_read,              //  3 - 0x03    0xFC
        intpr_CCP_write,             //  4 - 0x04    0xFB
        intpr_AFE_read,              //  5 - 0x05    0xFA
        intpr_AFE_write,             //  6 - 0x06    0xF9
        intpr_WUP_read,              //  7 - 0x07    0xF8
        intpr_WUP_write,             //  8 - 0x08    0xF7
        streaming_start,             //  9 - 0x09    0xF6
        fndummy,                     // 10 - 0x0A    0xF5 USB_ENG_Read  (DEPRECATED )
        fndummy,                     // 11 - 0x0B    0xF4 USB_ENG_Write (DEPRECATED )
        status_start,                // 12 - 0x0C    0xF3
        status_stop,                 // 13 - 0x0D    0xF2
        goto_sleep,                  // 14 - 0x0E    0xF1
        wake_up,                     // 15 - 0x0F    0xF0
        reset_soft,                  // 16 - 0x10    0xEF
        reset_hard,                  // 17 - 0x11    0xEE
        reset_hardware,              // 18 - 0x12    0xED
        sampling_rate_read,          // 19 - 0x13    0xEC
        sampling_rate_write,         // 20 - 0x14    0xEB
        logical_channels_read,       // 21 - 0x15    0xEA
        logical_channels_write,      // 22 - 0x16    0xE9
        set_emitter_timings,         // 23 - 0x17    0xE8
        set_emitter_state,           // 24 - 0x18    0xE7
        get_failure_flags,           // 25 - 0x19    0xE6
        board_type_read,             // 26 - 0x1A    0xE5
        unit_id_read,                // 27 - 0x1B    0xE4
        unit_id_write,               // 28 - 0x1C    0xE3
        second_unit_id_read,         // 29 - 0x1D    0xE2
        get_darkpixel_state,         // 30 - 0x1E    0xE1
        set_darkpixel_state,         // 31 - 0x1F    0xE0
        streaming_stop,              // 32 - 0x20    0xDF -- Received as code 0x77
        report_error,                // 33 - 0x21    0xDE
        fndummy                      // 34 - 0x22    0xDD
    };
    decode_result_type unpack_result = message_unpack_decode();
    if (unpack_result == no_data)
    {
        *cmd_code = 0x22;  // dummy function: no action
        next_step = execute_contact_gui;
    }
    if (unpack_result == decode_fail)
    {
        (*pf[0x21])();   // call report error
        *cmd_code = 0x21;
    }
    if (unpack_result == decode_success)
    {
        message_unpack_result(&cmd_tab_idx, destination, message_to_sensor_payload);
        sensor_destination = *destination;
        if (cmd_tab_idx < CMD_ARRAY_SIZE)
        {
            (*pf[cmd_tab_idx])();
        }
        *cmd_code = cmd_tab_idx;
    }
    return next_step;
}
/** end of message_interpreter
  * @}
  */
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
